//
//  mraid.js
//  MRAID
//
//  Created by Mobeleader on 13/01/2017.
//  Copyright (c) 2017 Beleader Internet Marketing S.L. All rights reserved.
//
(function() {

var mraid = window.mraid = {};

/***************************************************************************
 * constants
 **************************************************************************/

var VERSION = "2.0";

var STATES = mraid.STATES = {
    "LOADING" : "loading",
    "DEFAULT" : "default",
    "EXPANDED" : "expanded",
    "RESIZED" : "resized",
    "HIDDEN" : "hidden"
};

var PLACEMENT_TYPES = mraid.PLACEMENT_TYPES = {
    "INLINE" : "inline",
    "INTERSTITIAL" : "interstitial"
};

var RESIZE_PROPERTIES_CUSTOM_CLOSE_POSITION = mraid.RESIZE_PROPERTIES_CUSTOM_CLOSE_POSITION = {
    "TOP_LEFT" : "top-left",
    "TOP_CENTER" : "top-center",
    "TOP_RIGHT" : "top-right",
    "CENTER" : "center",
    "BOTTOM_LEFT" : "bottom-left",
    "BOTTOM_CENTER" : "bottom-center",
    "BOTTOM_RIGHT" : "bottom-right"
};

var ORIENTATION_PROPERTIES_FORCE_ORIENTATION = mraid.ORIENTATION_PROPERTIES_FORCE_ORIENTATION = {
    "PORTRAIT" : "portrait",
    "LANDSCAPE" : "landscape",
    "NONE" : "none"
};

var EVENTS = mraid.EVENTS = {
    "ERROR" : "error",
    "READY" : "ready",
    "SIZECHANGE" : "sizeChange",
    "STATECHANGE" : "stateChange",
    "VIEWABLECHANGE" : "viewableChange"
};

var SUPPORTED_FEATURES = mraid.SUPPORTED_FEATURES = {
    "SMS" : "sms",
    "TEL" : "tel",
    "CALENDAR" : "calendar",
    "STOREPICTURE" : "storePicture",
    "INLINEVIDEO" : "inlineVideo"
};

/***************************************************************************
 * state
 **************************************************************************/

var state = STATES.LOADING;
var placementType = PLACEMENT_TYPES.INLINE;
var supportedFeatures = {};
var isViewable = false;
var isResizable = false;

var expandProperties = {
    "width" : 0,
    "height" : 0,
    "useCustomClose" : false,
    "isModal" : true
};

var orientationProperties = {
    "allowOrientationChange" : true,
    "forceOrientation" : ORIENTATION_PROPERTIES_FORCE_ORIENTATION.NONE
};

var resizeProperties = {
    "width" : 0,
    "height" : 0,
    "customClosePosition" : RESIZE_PROPERTIES_CUSTOM_CLOSE_POSITION.TOP_RIGHT,
    "offsetX" : 0,
    "offsetY" : 0,
    "allowOffscreen" : true
};

var currentPosition = {
    "x" : 0,
    "y" : 0,
    "width" : 0,
    "height" : 0
};

var defaultPosition = {
    "x" : 0,
    "y" : 0,
    "width" : 0,
    "height" : 0
};

var maxSize = {
    "width" : 0,
    "height" : 0
};

var screenSize = {
    "width" : 0,
    "height" : 0
};

 var currentOrientation = 0;

 var listeners = {};
 
 var validators = {
    "setExpandProperties": {
        "width" : function(width) {
            return !isNaN(width);
        },
        "height" : function(height) {
            return !isNaN(height);
        },
        "useCustomClose" : function(useCustomClose) {
            return (typeof useCustomClose === "boolean");
        }
    },
    "setResizeProperties": {
        "width" : function(width) {
            return !isNaN(width) && width >= 50;
        },
        "height" : function(height) {
            return !isNaN(height) && height >= 50;
        },
        "offsetX" : function(offsetX) {
            return !isNaN(offsetX);
        },
        "offsetY" : function(offsetY) {
            return !isNaN(offsetY);
        },
        "customClosePosition" : function(customClosePosition) {
            var positions = [ "top-left","top-center","top-right","center","bottom-left","bottom-center","bottom-right" ];
            return positions.indexOf(customClosePosition) !== -1;
        },
        "allowOffscreen" : function(allowOffscreen) {
            return (typeof allowOffscreen === "boolean");
        }
    },
    "setOrientationProperties": {
        "allowOrientationChange" : function(allowOrientationChange) {
            return (typeof allowOrientationChange === "boolean");
        },
        "forceOrientation" : function(forceOrientation) {
            var values = [ "portrait","landscape","none" ];
            return values.indexOf(forceOrientation) !== -1;
        }
    }
 };

// "official" API: methods called by creative

mraid.addEventListener = function(event, listener) {
    if (!event || !listener) {
        mraid.triggerErrorEvent("Both event and listener are required.", "addEventListener");
        return;
    }
    if (!eventContained(event, EVENTS)) {
        mraid.triggerErrorEvent("Unknown MRAID event: " + event, "addEventListener");
        return;
    }
    var listenersForEvent = listeners[event] = listeners[event] || [];
    for (var i = 0; i < listenersForEvent.length; i++) {
        var str1 = String(listener);
        var str2 = String(listenersForEvent[i]);
        if (listener === listenersForEvent[i] || str1 === str2) {
            mraid.triggerErrorEvent("MRAID event is already added");
            return;
        }
    }
    listenersForEvent.push(listener);
};

mraid.close = function() {
    mraid.log("Placement type is '" + placementType + "' and custom close is '" + expandProperties.useCustomClose + "'");
    if (placementType === PLACEMENT_TYPES.INLINE && !expandProperties.useCustomClose) {
        mraid.log("Placement type is 'inline' and creative is not using custom close", 5);
        return
    }
    if (state === STATES.DEFAULT) {
        mraid.triggerStateChangeEvent("hidden");
    }
    else if (state === STATES.EXPANDED || state === STATES.RESIZED) {
        mraid.triggerStateChangeEvent("default");
    }
    callNative("close");
};

mraid.expand = function(url) {
    if (placementType !== PLACEMENT_TYPES.INLINE || (state !== STATES.DEFAULT && state !== STATES.RESIZED)) {
        mraid.triggerErrorEvent("MRAID cannot expand: state is '" + state + "' and placementType is '" + placementType + "'", "addEventListener");
        return;
    }
    if (url === undefined) {
        callNative("expand");
    } else {
        callNative("expand?url=" + encodeURIComponent(url));
    }
};

mraid.getCurrentPosition = function() {
    return currentPosition;
};

mraid.getDefaultPosition = function() {
    return defaultPosition;
};

mraid.getExpandProperties = function() {
    return expandProperties;
};

mraid.getMaxSize = function() {
    return maxSize;
};

mraid.getOrientationProperties = function() {
    return orientationProperties;
};

mraid.getPlacementType = function() {
    return placementType;
};

mraid.getResizeProperties = function() {
    return resizeProperties;
};

mraid.getScreenSize = function() {
    return screenSize;
};

mraid.getState = function() {
    return state;
};

mraid.getVersion = function() {
    return VERSION;
};

mraid.isViewable = function() {
    return isViewable;
};

mraid.open = function(url) {
    callNative("open?url=" + encodeURIComponent(url));
};

mraid.playVideo = function(url) {
    callNative("playVideo?url=" + encodeURIComponent(url));
};

mraid.removeEventListener = function(event, listener) {
    if (!event) {
        mraid.triggerErrorEvent("Event is required.", "removeEventListener");
        return;
    }
    if (!eventContained(event, EVENTS)) {
        mraid.triggerErrorEvent("Unknown MRAID event: " + event, "removeEventListener");
        return;
    }
    if (listeners.hasOwnProperty(event)) {
        if (listener) {
            var listenersForEvent = listeners[event];
            var len = listenersForEvent.length;
            for (var i = 0; i < len; i++) {
                var registeredListener = listenersForEvent[i];
                var str1 = String(listener);
                var str2 = String(registeredListener);
                if (listener === registeredListener || str1 === str2) {
                    listenersForEvent.splice(i, 1);
                    break;
                }
            }
            if (listenersForEvent.length === 0) {
                delete listeners[event];
            }
        } else {
            delete listeners[event];
        }
    }
};

 mraid.resize = function() {
    if (!isResizable) {
        mraid.triggerErrorEvent("Creative cannot be resized. Resize properties are not set", "mraid.resize");
        return;
    }
    if (placementType === PLACEMENT_TYPES.INTERSTITIAL) {
        mraid.triggerErrorEvent("Creative cannot be resized when placement type is '" + placementType + "'", "mraid.resize");
        return;
    }
    if (state === STATES.EXPANDED || state === STATES.LOADING || state === STATES.HIDDEN) {
        mraid.triggerErrorEvent("Creative cannot be resized when state is '" + state + "'", "mraid.resize");
        return;
    }
    callNative("resize");
};

mraid.setExpandProperties = function(properties) {
    if (state === STATES.EXPANDED) {
        mraid.log("Expand properties cannot be set. Creative state is already 'expanded'", 5);
        return;
    }
    if (!checkProperties(properties, "setExpandProperties")) {
        mraid.triggerErrorEvent("MRAID cannot validate expand properties", "addEventListener");
        return;
	}
    var oldUseCustomClose = expandProperties.useCustomClose;
    var props = [ "width", "height", "useCustomClose" ];
    for (var i = 0; i < props.length; i++) {
        var propname = props[i];
        if (properties.hasOwnProperty(propname)) {
            expandProperties[propname] = properties[propname];
        }
    }
    if (expandProperties.useCustomClose !== oldUseCustomClose) {
        callNative("useCustomClose?useCustomClose=" + expandProperties.useCustomClose);
    }
};

mraid.setOrientationProperties = function(properties) {
    if (!checkProperties(properties, "setOrientationProperties")) {
        mraid.triggerErrorEvent("MRAID cannot validate orientation properties", "addEventListener");
        return;
	}
    
    var newOrientationProperties = {};
    newOrientationProperties.allowOrientationChange = orientationProperties.allowOrientationChange,
    newOrientationProperties.forceOrientation = orientationProperties.forceOrientation;
    
    var rwProps = [ "allowOrientationChange", "forceOrientation" ];
    for (var i = 0; i < rwProps.length; i++) {
        var propname = rwProps[i];
        if (properties.hasOwnProperty(propname)) {
            newOrientationProperties[propname] = properties[propname];
        }
    }
    
    if (newOrientationProperties.allowOrientationChange &&
        newOrientationProperties.forceOrientation !== mraid.ORIENTATION_PROPERTIES_FORCE_ORIENTATION.NONE) {
        var msg = "allowOrientationChange is true but forceOrientation is " + newOrientationProperties.forceOrientation;
        mraid.log(msg,5);
        return;
    }
    
    orientationProperties.allowOrientationChange = newOrientationProperties.allowOrientationChange;
    orientationProperties.forceOrientation = newOrientationProperties.forceOrientation;
    
    var params =
    "allowOrientationChange=" + orientationProperties.allowOrientationChange +
    "&forceOrientation=" + orientationProperties.forceOrientation;
    
    callNative("setOrientationProperties?" + params);
};
 
mraid.setResizeProperties = function(properties) {
    isResizable = false;
    var requiredProps = [ "width", "height", "offsetX", "offsetY" ];
    for (var i = 0; i < requiredProps.length; i++) {
        var propname = requiredProps[i];
        if (!properties.hasOwnProperty(propname)) {
            mraid.triggerErrorEvent("required property " + propname + " is missing", "mraid.setResizeProperties");
            return;
        }
    }
    if (!checkProperties(properties, "setResizeProperties")) {
        mraid.triggerErrorEvent("failed validation", "mraid.setResizeProperties");
        return;
    }
    var allowOffscreen = properties.hasOwnProperty("allowOffscreen") ? properties.allowOffscreen : resizeProperties.allowOffscreen;
    if (!allowOffscreen) {
        if (properties.width > maxSize.width || properties.height > maxSize.height) {
            mraid.triggerErrorEvent("Resize property allowOffscreen is false and resize dimensions excede creative maxSize", "mraid.setResizeProperties");
            return;
        }
        properties = checkResizeProperties(properties);
    }
    else if (!closeRegionFitsOnScreen(properties)) {
        mraid.triggerErrorEvent("Close event region will not appear entirely onscreen", "mraid.setResizeProperties");
        return;
    }
    var rwProps = [ "width", "height", "offsetX", "offsetY", "customClosePosition", "allowOffscreen" ];
    for (var i = 0; i < rwProps.length; i++) {
        var propname = rwProps[i];
        if (properties.hasOwnProperty(propname)) {
            resizeProperties[propname] = properties[propname];
        }
    }
    var params = "width=" + resizeProperties.width + "&height=" + resizeProperties.height + "&offsetX=" + resizeProperties.offsetX + "&offsetY=" + resizeProperties.offsetY + "&customClosePosition=" + resizeProperties.customClosePosition + "&allowOffscreen=" + resizeProperties.allowOffscreen;
    callNative("setResizeProperties?" + params);
    isResizable = true;
 };

mraid.storePicture = function(url) {
    callNative("storePicture?url=" + encodeURIComponent(url));
};

mraid.supports = function(feature) {
    var val = supportedFeatures[feature];
    if (typeof val === "undefined") {
        val = false;
    }
    return val;
};

mraid.useCustomClose = function(isCustomClose) {
    if (expandProperties.useCustomClose !== isCustomClose) {
        expandProperties.useCustomClose = isCustomClose;
        callNative("useCustomClose?useCustomClose=" + expandProperties.useCustomClose);
    }
 };
 
 mraid.createCalendarEvent = function(parameters) {
    callNative("createCalendarEvent?eventJSON="+JSON.stringify(parameters));
 };

// setters from native to change properties

mraid.setCurrentPosition = function(x, y, width, height) {
    var previousWidth = currentPosition.width;
    var previousHeight = currentPosition.height;
    currentPosition.x = x;
    currentPosition.y = y;
    currentPosition.width = width;
    currentPosition.height = height;
    if (width !== previousWidth || height !== previousWidth) {
        mraid.triggerSizeChangeEvent(width, height);
    }
};

mraid.setDefaultPosition = function(x, y, width, height) {
    defaultPosition.x = x;
    defaultPosition.y = y;
    defaultPosition.width = width;
    defaultPosition.height = height;
};

mraid.setExpandSize = function(width, height) {
    expandProperties.width = width;
    expandProperties.height = height;
};

mraid.setMaxSize = function(width, height) {
    maxSize.width = width;
    maxSize.height = height;
};

mraid.setPlacementType = function(pt) {
    placementType = pt;
};

mraid.setScreenSize = function(width, height) {
    screenSize.width = width;
    screenSize.height = height;
};

mraid.setSupports = function(feature, supported) {
    supportedFeatures[feature] = supported;
};

// methods to trigger events

mraid.triggerErrorEvent = function(message, action) {
    mraid.log(message,5);
    triggerEvent(mraid.EVENTS.ERROR, message, action);
};

mraid.triggerReadyEvent = function() {
    triggerEvent(mraid.EVENTS.READY);
    mraid.log("Creative is ready", 3);
};

mraid.triggerSizeChangeEvent = function(width, height) {
    triggerEvent(mraid.EVENTS.SIZECHANGE, width, height);
    mraid.log("Creative size changed, width=" + width + ", height=" + height, 3);
};

mraid.triggerStateChangeEvent = function(newState) {
    if (state !== newState) {
        state = newState;
        triggerEvent(mraid.EVENTS.STATECHANGE, state);
        mraid.log("Creative state changed to " + state, 3);
    }
};

mraid.triggerViewableChangeEvent = function(newIsViewable) {
    if (isViewable !== newIsViewable) {
        isViewable = newIsViewable;
        triggerEvent(mraid.EVENTS.VIEWABLECHANGE, isViewable);
        mraid.log("Creative changed visibility to " + isViewable, 3);
    }
};

/***************************************************************************
 * internal helper methods
 **************************************************************************/

var callNative = function(command) {
    var iframe = document.createElement("IFRAME");
    iframe.setAttribute("src", "mraid://" + command);
    document.documentElement.appendChild(iframe);
    iframe.parentNode.removeChild(iframe);
    iframe = null;
};

var triggerEvent = function(event) {
    var args = Array.prototype.slice.call(arguments);
    args.shift();
    var eventListeners = listeners[event];
    if (eventListeners) {
        var len = eventListeners.length;
        for (var i = 0; i < len; i++) {
            eventListeners[i].apply(null, args);
        }
    }
};

var eventContained = function(value, array) {
    for (var i in array) {
        if (array[i] === value) {
            return true;
        }
    }
    return false;
};

function closeRegionFitsOnScreen(properties) {
 
    var customClosePosition = properties.hasOwnProperty("customClosePosition") ? properties.customClosePosition : resizeProperties.customClosePosition;
 
    var customCloseSize = { "width": 50, "height": 50 };
 
    if (customClosePosition.search("left") !== -1) {
        customCloseSize.x = defaultPosition.x + properties.offsetX;
    }
    else if (customClosePosition.search("center") !== -1) {
        customCloseSize.x = defaultPosition.x + properties.offsetX + (properties.width / 2) - 25;
    }
    else if (customClosePosition.search("right") !== -1) {
        customCloseSize.x = defaultPosition.x + properties.offsetX + properties.width - 50;
    }
 
    if (customClosePosition.search("top") !== -1) {
        customCloseSize.y = defaultPosition.y + properties.offsetY;
    }
    else if (customClosePosition === "center") {
        customCloseSize.y = defaultPosition.y + properties.offsetY + (properties.height / 2) - 25;
    }
    else if (customClosePosition.search("bottom") !== -1) {
        customCloseSize.y = defaultPosition.y + properties.offsetY + properties.height - 50;
    }
 
    var container = { "x": 0, "y": 0 };
    container.width = maxSize.width;
    container.height = maxSize.height;
 
    return isRectContained(container, customCloseSize);
}
 
 function checkResizeProperties(properties) {
    var resizeRect = {};
    resizeRect.x = defaultPosition.x + properties.offsetX;
    resizeRect.y = defaultPosition.y + properties.offsetY;
    resizeRect.width = properties.width;
    resizeRect.height = properties.height;
 
    var maxRect = { "x": 0, "y": 0 };
    maxRect.width = maxSize.width;
    maxRect.height = maxSize.height;
 
    if (resizeRect.x < maxRect.x) {
        properties.offsetX = maxRect.x - defaultPosition.x;
    }
    else if ((resizeRect.x + resizeRect.width) > maxRect.width) {
        properties.offsetX = maxRect.width - (defaultPosition.x + properties.width);
    }
 
    if (resizeRect.y < maxRect.y) {
        properties.offsetY = maxRect.y - defaultPosition.y;
    }
    else if ((resizeRect.y + resizeRect.height) > maxRect.height) {
        properties.offsetY = maxRect.height - (defaultPosition.y + properties.height);
    }
 
    return properties;
 }
 
 function isRectContained(container, containedRect) {
    var containedRectTotalWidth = containedRect.x + containedRect.width;
    var containedRectTotalHeight = containedRect.y + containedRect.height;
    return (container.width >= containedRectTotalWidth && container.height >= containedRectTotalHeight);
 }
 
 function checkProperties(properties, action) {
    var isValid = true;
    var actionProperties = validators[action];
    for (var prop in properties) {
        var validator = actionProperties[prop];
        var value = properties[prop];
        if (validator && !validator(value)) {
            mraid.triggerErrorEvent("Value of property " + prop + " (" + value	+ ") is invalid", "mraid." + action);
            isValid = false;
        }
    }
    return isValid;
 }
 
 mraid.log = function(logMessage,level) {
    callNative("setMessage?message=" + encodeURIComponent(logMessage) + "&level=" + level);
 };

}());
